#include "BLG_user.h"
#define SVMTRUE 1
#define SVMFALSE 0

/* ========================================================================== */
/* === structures =========================================================== */
/* ========================================================================== */

typedef struct SVMparm_struct /* SVM parameters */
{
    int          nsub ; /* subspace size */
    double     PreAmp ; /* amplification factor for value of prior indices */
    double    MaxKept ; /* maximum fraction of prior indices in new space */
    /* subspace error tolerance = MAX (err1, err2) */
    double      serr1 ; /* err1 = serr1 times SVM tolerance */
    double      serr2 ; /* err2 = serr2 times prior KKT error */
    /* factor forcing at least one BLG iteration -- error tolerance is
       the force factor times the error at the current point in subspace */
    double   ForceFac ; /* factor < 1 forcing at least 1 BLG iteration */
    double     d_pert ; /* only include d_i in subspace if >= d_pert */
    double   bnd_pert ; /* threshhold for free variables is C*bnd_cut */
    double   lpfactor ; /* parameter in decision rule of QP versus LP */
    double      gamma ; /* parameter in kernel */
    short      degree ; /* degree in poly kernel */
    int      maxouter ; /* maximum number of outer iterations */
    int    PrintLevel ; /* 0 (none), 1 (error+cost), 2 (+statistics) */
    short   SparseDot ; /* 1 means by sparse dot product, 0 means by scatter */
    short     Ax_is_1 ; /* 1 means all element of Ax are 1, Ax ignored */
    short       FormS ; /* if kernel > 0 or FormS = 1, then dense S built */
    int    PrintParms ; /* 1 means print parameter values */
} SVMparm ;

typedef struct SVMstat_struct /* statistics returned to user */
{
    double          f ; /* function value */
    double        err ; /* max norm of g - mu d */
    int     outer_its ; /* number of subspaces */
    int     inner_its ; /* total number of iterations to solve subspace probs */
    int            nf ; /* total number of times subspace cost evaluated */
    int            ng ; /* total number of times subspace gradient evaluated */
    int         ncols ; /* number of columns in A evaluated */
    int           nmu ; /* total number of iterations in computation of mu */
    double   Subspace ; /* time spent in routine to construct subspace */
    double        BLG ; /* time spent in BLG routine solving subspace problem */
    double     BuildS ; /* time spent building subspace matrix S */
    double UpdateGrad ; /* time spent updating gradient */
} SVMstat ;

/* prototypes */

int SVMsubspace /* return:
                   0 (error tolerance satisfied)
                   1 (number of iterations exceeded limit in BLG or in SVM)
                   2 (out of memory)
                   3 (line search failed)
                   4 (search direction not a descent direction)
                   5 (computation of search direction fails)
                   6 (search direction not a descent direction )
                   7 (computation of mu fails)
                   8 (function value is nan in line search)*/
(
    double       *x, /* output: the solution */
    float       *Ax, /* numerical values in matrix */
    short       *Ai, /* row indices */
    int         *Ap, /* pointers to start of each column in Ax, Ai */
    short        *a, /* label vector (+- 1) */
    double        C, /* constraint is 0 <= x_i <= C */
    int           n, /* problem dimension */
    int    nfeature, /* number of features */
    short    kernel, /* 0 = linear, 1 = poly, 2 = rbf, 3 = sig */
    double      tol, /* bound on KKT error */
    SVMstat   *Stat, /* structure with statistics (can be NULL) */
    SVMparm  *sparm, /* SVM parameters, NULL = use default parameters */
    BLGparm  *bparm  /* BLG parameters, NULL = use default parameters */
) ;

void SVMdefault
(
    SVMparm *Parm, /* pointer to parameter structure */
    int  nfeature  /* number of features in the data */
) ;
